@@ -167,7 +167,7 @@ void myclock(int dummy)
 static char *buffer; /* The buffer is only growing and never freed... */
 static unsigned buffersize;
 
-static void buf_wr(unsigned idx, char val)
+static void buf_wr(unsigned idx, unsigned char val)
 {
   if (idx >= buffersize)
     {
@@ -1092,8 +1092,11 @@ int main(int argc, char **argv)
 #if 0 /* Shouldn't need this.. */
   signal(SIGHUP, SIG_IGN);
 #endif
-  /* On some Linux systems SIGALRM is masked by default. Unmask it */  
-  sigrelse(SIGALRM);  
+  /* On some Linux systems SIGALRM is masked by default. Unmask it */
+  sigset_t ss;
+  sigemptyset(&ss);
+  sigaddset(&ss, SIGALRM);
+  sigprocmask(SIG_UNBLOCK, &ss, NULL);
 
   /* initialize locale support */
   setlocale(LC_ALL, "");
