@@ -385,8 +385,8 @@ void updown(int what, int nr)
             trim (trimbuf, buf, sizeof(trimbuf));
             do_log("%s", trimbuf);
           } else if (!strncmp (buffirst, "Bytes", 5)) {
-            strncpy (xfrstr, buf, sizeof(xfrstr));
-            xfrstr[sizeof(xfrstr) - 1] = '\0';
+            _Static_assert(sizeof(xfrstr) >= sizeof(buf), "String sizes");
+            strcpy(xfrstr, buf);
           }
           buffirst[0] = 0;
           trimbuf[0] = 0;
@@ -428,10 +428,13 @@ void updown(int what, int nr)
   timer_update();
 
   /* If we got interrupted, status != 0 */
-  if (win && (status & 0xFF00) == 0) {
+  if (win && ((status & 0xFF00) == 0 || status == 1)) {
 #if VC_MUSIC
     if (P_SOUND[0] == 'Y') {
-      mc_wprintf(win, _("\n READY: press any key to continue..."));
+      if (status == 1)
+        mc_wprintf(win, _("\n Failure executing protocol. Press any key to continue..."));
+      else
+        mc_wprintf(win, _("\n READY: press any key to continue..."));
       music();
     } else
       sleep(1);
@@ -811,7 +814,6 @@ int paste_file(void)
   char buf[128] = "";
   char *ptr;
   int bytes_read;
-  unsigned long bdone = 0;
   int x;
 
   if ((s = filedir(1, 0)) == NULL)
@@ -845,11 +847,9 @@ int paste_file(void)
 	vt_send(*s);
       vt_send('\r');
       vt_send('\n');
-      bdone += strlen(line) + 2;
     } else {
       for (s = line; *s; s++)
 	vt_send(*s);
-      bdone += strlen(s);
     }
     if (ldelay) {
 #ifdef HAVE_USLEEP
